import inspect
import logging
import queue
import logging.handlers
from . import utils

loggers = {}

def setup_custom_logger(name):
    global loggers

    if loggers.get(name):
        return loggers.get(name)
    else:
        formatter = logging.Formatter(
            "%(asctime)s [%(module)s:%(lineno)d] %(levelname)s %(message)s"
        )

        logger = logging.getLogger(name)
        # clean the handlers, otherwise you get duplicated records when logging
        if logger.hasHandlers():
            return logger
            # logger.handlers.clear()
        logger.propagate = False
        level = logging.getLevelName(utils.get_logging_level())
        logger.setLevel(level)
        log_queue = queue.Queue()
        queue_handler = logging.handlers.QueueHandler(log_queue)
        # set the non-blocking handler first
        logger.addHandler(queue_handler)

        # Stream is important for looking at the k8s pod logs.
        stream_handler = logging.StreamHandler()
        stream_handler.setLevel(logging.DEBUG)
        stream_handler.setFormatter(formatter)

        # the local logging is fine, but there sould be a Fluent Bit integration, which sends logs to the central LMM instance
        timerotating_handler = logging.handlers.TimedRotatingFileHandler(
            utils.get_log_path().joinpath("app_rolling.log"), when="D", backupCount=30
        )
        timerotating_handler.setLevel(utils.get_logging_level())
        timerotating_handler.setFormatter(formatter)
        listener = logging.handlers.QueueListener(
            log_queue, stream_handler, timerotating_handler, respect_handler_level=True
        )

        # Only print the following when instantiated by the main.py file - and not other files.
        # This will ensure that the important project variables are printet on startup.
        current_stack = inspect.stack()
        if any("main.py" in frame.filename for frame in current_stack):
            # Print settings:
            logger.info(f"Starting {utils.get_application_name()} in {utils.get_environment()}.")
            logger.info(f"Domain {utils.get_domain_name()}")
            logger.info(f"Root {utils.get_project_root()}")
            logger.info(f"Log Path {utils.get_log_path()}")
            logger.info(f"Logging Level {utils.get_logging_level()}")

        listener.start()

    return logger



import ipaddress
import email_validator

def validate_email(email: str):
    email_validator.validate_email(email, check_deliverability=True)

def validate_ip_address(ip_string: str):
    ipaddress.ip_address(ip_string)




import os
import pathlib
import pytest
import yaml
from tksessentials import utils

@pytest.fixture
def mock_project_root(monkeypatch, tmp_path):
    # Directly set the PROJECT_ROOT in the utils module
    utils.PROJECT_ROOT = tmp_path
    return tmp_path

def test_get_project_root(mock_project_root):
    expected_path = str(mock_project_root)
    assert utils.get_project_root() == expected_path


def test_get_project_root_path(mock_project_root):
    assert utils.get_project_root_path() == mock_project_root


def test_get_app_config(monkeypatch, tmp_path):
    # Create a temporary config directory
    config_dir = tmp_path / "config"
    config_dir.mkdir()
    mock_config_file = config_dir / "app_config.yaml"

    # Write the mock configuration
    expected_config = {
        "application": "TestApp",
        "domain": "test.domain.com",
        "env": "DEV",
        "logging_level": "INFO"
    }
    with open(mock_config_file, "w") as file:
        yaml.dump(expected_config, file)

    # Override the `find_project_root` function to return the tmp_path
    def mock_find_project_root(*args, **kwargs):
        return tmp_path

    monkeypatch.setattr(utils, "find_project_root", mock_find_project_root)

    # Run the test
    assert utils.get_app_config() == expected_config

def test_find_project_root():
    try:        
        project_root = utils.find_project_root(pathlib.Path(__file__).resolve())
        print(project_root)
    except Exception as e:
        # If any exception is caught, fail the test explicitly.
        assert False, f"An exception was thrown: {e}"

def test_get_application_name_success(monkeypatch):
    def mock_get_app_config():
        return {"application": "TestApp"}

    monkeypatch.setattr(utils, "get_app_config", mock_get_app_config)
    assert utils.get_application_name() == "TestApp"

def test_get_application_name_failure(monkeypatch):
    def mock_get_app_config():
        return {}

    monkeypatch.setattr(utils, "get_app_config", mock_get_app_config)
    with pytest.raises(ValueError):
        utils.get_application_name()

def test_get_domain_name_success(monkeypatch):
    def mock_get_app_config():
        return {"domain": "test.domain.com"}

    monkeypatch.setattr(utils, "get_app_config", mock_get_app_config)
    assert utils.get_domain_name() == "test.domain.com"

def test_get_domain_name_failure(monkeypatch):
    def mock_get_app_config():
        return {}

    monkeypatch.setattr(utils, "get_app_config", mock_get_app_config)
    with pytest.raises(ValueError):
        utils.get_domain_name()



import time
import pytest
from tksessentials import database, utils
from tksessentials.database import KSQLNotReadyError
from unittest.mock import patch
import httpx
import asyncio
from kafka.admin import KafkaAdminClient, NewTopic
from kafka import KafkaConsumer
from kafka.errors import TopicAlreadyExistsError

@pytest.fixture(scope="module")
def setup():
    print("\n")
    print("REMINDER. All tests require a Kafka broker and a ksqlDB instance to be available.")
    print(f"\tENV: {utils.get_environment()}")
    print(f"\tkafka: {database.get_kafka_cluster_brokers()}")
    print(f"\tksqldb: {database.get_ksqldb_url()}")

def test_get_kafka_cluster_brokers_dev():
    # Mock utils.get_environment() to return 'DEV'
    with patch('tksessentials.utils.get_environment', return_value='DEV'):
        brokers = database.get_kafka_cluster_brokers()
        # Check if the returned brokers are as expected
        assert brokers == ['localhost:9092']

def test_get_kafka_cluster_brokers_non_dev():
    # Mock utils.get_environment() to return 'PROD'
    with patch('tksessentials.utils.get_environment', return_value='PROD'):
        # Mock os.getenv to return a specific broker string
        with patch('os.getenv', return_value='broker1.svc.cluster.local:9092,broker2.svc.cluster.local:9093'):
            brokers = database.get_kafka_cluster_brokers()
            # Check if the returned brokers contain the expected substring
            assert 'broker1.svc.cluster.local:9092' in brokers
            assert brokers == ['broker1.svc.cluster.local:9092', 'broker2.svc.cluster.local:9093']

def test_table_or_view_exists():
    max_retries = 5
    delay = 10  # seconds

    for attempt in range(max_retries):
        try:
            assert not database.table_or_view_exists("NON_EXISTENT_TABLE")
            break
        except (
            httpx.ConnectError,
            KSQLNotReadyError,
            httpx.RemoteProtocolError,
            httpx.ReadError,
        ) as e:
            if attempt < max_retries - 1:
                print(
                    f"Attempt {attempt + 1}/{max_retries} failed with error: {e}. Retrying in {delay} seconds..."
                )
                time.sleep(delay)
            else:
                raise
        except Exception as e:
            print(f"Test failed with an unexpected error: {e}. {e.args}")
            raise

@pytest.mark.asyncio
async def test_create_table():
    TABLE_NAME = "TEST_TABLE"
    sql_statement = f"""
    CREATE TABLE {TABLE_NAME}(
        event_timestamp BIGINT PRIMARY KEY,
        detail STRING,
        data STRING
    ) WITH (
        KAFKA_TOPIC='test_topic',
        VALUE_FORMAT='JSON',
        PARTITIONS=1
    );
    """

    max_retries = 5
    delay = 10  # seconds

    for attempt in range(max_retries):
        try:
            await database.create_table(sql_statement, TABLE_NAME)
            assert database.table_or_view_exists(TABLE_NAME)
            break
        except (
            httpx.ConnectError,
            KSQLNotReadyError,
            httpx.RemoteProtocolError,
            httpx.ReadError,
        ) as e:
            if attempt < max_retries - 1:
                print(
                    f"Attempt {attempt + 1}/{max_retries} failed with error: {e}. Retrying in {delay} seconds..."
                )
                time.sleep(delay)
            else:
                raise
        except Exception as e:
            print(f"Test failed with an unexpected error: {e}")
            raise

@pytest.mark.asyncio
async def test_prepare_sql_statement():
    sql_statement = """
    CREATE TABLE TEST_TABLE_PARTITION(
        event_timestamp BIGINT PRIMARY KEY,
        detail STRING,
        data STRING,
        signal_data STRUCT<
            provider_signal_id STRING,
            provider_trade_id STRING,
            provider_id STRING,
            strategy_id STRING,
            is_hot_signal BOOLEAN,
            market STRING,
            data_source STRING,
            direction STRING,
            side STRING,
            order_type STRING,
            price DOUBLE,
            tp DOUBLE,
            sl DOUBLE,
            position_size_in_percentage INT,
            date_of_creation BIGINT
        >,
        ip STRING
    ) WITH (
        KAFKA_TOPIC='test_topic_partition',
        VALUE_FORMAT='JSON',
        TIMESTAMP='event_timestamp',
        PARTITIONS=3
    );
    """

    with patch('tksessentials.database.topic_exists', return_value=True):
        prepared_statement = await database.prepare_sql_statement(sql_statement)
        assert "PARTITIONS=3" not in prepared_statement

    with patch('tksessentials.database.topic_exists', return_value=False):
        prepared_statement = await database.prepare_sql_statement(sql_statement)
        assert "PARTITIONS=3" in prepared_statement

@pytest.mark.asyncio
async def test_prepare_sql_statement_missing():
    """This test checks if the preparation works as well, wenn there is no PARTITION?"""
    sql_statement = """
    CREATE TABLE TEST_TABLE_PARTITION(
        event_timestamp BIGINT PRIMARY KEY,
        detail STRING,
        data STRING,
        signal_data STRUCT<
            provider_signal_id STRING,
            provider_trade_id STRING,
            provider_id STRING,
            strategy_id STRING,
            is_hot_signal BOOLEAN,
            market STRING,
            data_source STRING,
            direction STRING,
            side STRING,
            order_type STRING,
            price DOUBLE,
            tp DOUBLE,
            sl DOUBLE,
            position_size_in_percentage INT,
            date_of_creation BIGINT
        >,
        ip STRING
    ) WITH (
        KAFKA_TOPIC='test_topic_partition',
        VALUE_FORMAT='JSON',
        TIMESTAMP='event_timestamp'
    );
    """

    with patch('tksessentials.database.topic_exists', return_value=True):
        prepared_statement = await database.prepare_sql_statement(sql_statement)
        assert "PARTITIONS=3" not in prepared_statement

    with patch('tksessentials.database.topic_exists', return_value=False):
        prepared_statement = await database.prepare_sql_statement(sql_statement)
        assert "PARTITIONS=3" in prepared_statement

def test_clean_sql_statement():
    sql_statement = """
    CREATE TABLE fa_signal_processing_trading_signal_received(
        event_timestamp BIGINT PRIMARY KEY,
        detail STRING,
        data STRING,
        signal_data STRUCT<
            provider_signal_id STRING,
            provider_trade_id STRING,
            provider_id STRING,
            strategy_id STRING,
            is_hot_signal BOOLEAN,
            market STRING,
            data_source STRING,
            direction STRING,
            side STRING,
            order_type STRING,
            price DOUBLE,
            tp DOUBLE,
            sl DOUBLE,
            position_size_in_percentage INT,
            date_of_creation BIGINT
        >,
        ip STRING
    ) WITH (
        KAFKA_TOPIC='fa_signal_processing.trading_signal_received',
        VALUE_FORMAT='JSON',
        TIMESTAMP='event_timestamp',
        PARTITIONS=3
    );
    """

    expected_cleaned_sql = "CREATE TABLE fa_signal_processing_trading_signal_received( event_timestamp BIGINT PRIMARY KEY, detail STRING, data STRING, signal_data STRUCT< provider_signal_id STRING, provider_trade_id STRING, provider_id STRING, strategy_id STRING, is_hot_signal BOOLEAN, market STRING, data_source STRING, direction STRING, side STRING, order_type STRING, price DOUBLE, tp DOUBLE, sl DOUBLE, position_size_in_percentage INT, date_of_creation BIGINT >, ip STRING ) WITH ( KAFKA_TOPIC='fa_signal_processing.trading_signal_received', VALUE_FORMAT='JSON', TIMESTAMP='event_timestamp', PARTITIONS=3 );"

    cleaned_sql = database.clean_sql_statement(sql_statement)
    assert cleaned_sql == expected_cleaned_sql

def test_stream_exists():
    max_retries = 5
    delay = 10  # seconds

    for attempt in range(max_retries):
        try:
            assert not database.stream_exists("NON_EXISTENT_STREAM")
            break
        except (
            httpx.ConnectError,
            KSQLNotReadyError,
            httpx.RemoteProtocolError,
            httpx.ReadError,
        ) as e:
            if attempt < max_retries - 1:
                print(
                    f"Attempt {attempt + 1}/{max_retries} failed with error: {e}. Retrying in {delay} seconds..."
                )
                time.sleep(delay)
            else:
                raise
        except Exception as e:
            print(f"Test failed with an unexpected error: {e}. {e.args}")
            raise

@pytest.mark.asyncio
async def test_create_stream():
    STREAM_NAME = "TEST_STREAM"
    sql_statement = f"""
    CREATE STREAM {STREAM_NAME}(
        event_timestamp BIGINT,
        detail STRING,
        data STRING
    ) WITH (
        KAFKA_TOPIC='test_topic',
        VALUE_FORMAT='JSON',
        PARTITIONS=1
    );
    """

    max_retries = 5
    delay = 10  # seconds

    for attempt in range(max_retries):
        try:
            await database.create_stream(sql_statement, STREAM_NAME)
            assert database.stream_exists(STREAM_NAME)
            break
        except (
            httpx.ConnectError,
            KSQLNotReadyError,
            httpx.RemoteProtocolError,
            httpx.ReadError,
        ) as e:
            if attempt < max_retries - 1:
                print(
                    f"Attempt {attempt + 1}/{max_retries} failed with error: {e}. Retrying in {delay} seconds..."
                )
                time.sleep(delay)
            else:
                raise
        except Exception as e:
            print(f"Test failed with an unexpected error: {e}")
            raise


import re
from typing import Set
from . import global_logger

class AssetFormatter:
    def __init__(self, default_quote_asset: str = "USDT") -> None:
        self.default_quote_asset = default_quote_asset
        self.logger = global_logger.setup_custom_logger("app")
        self.symbol_pairs = {"WETH": "ETH"}

    def unwrap_symbol(self, wrapped_symbol):
        # Convert the input to uppercase to make the function case-insensitive
        wrapped_symbol_upper = wrapped_symbol.upper()
        # Return the unwrapped symbol, or the original symbol if not found
        return self.symbol_pairs.get(wrapped_symbol_upper, wrapped_symbol)

    def get_wrapped_symbol(self, original_symbol) -> str:
        """Returns the wrapped symbol for an asset. Or None"""
        for key, val in self.symbol_pairs.items():
            if val == original_symbol:
                return key
        return None

    def unwrap_market(self, market):
        # Extract the base and quote assets from the market string
        base_asset = self.get_base(market)
        quote_asset = self.get_quote(market)
        unwrapped_base = self.unwrap_symbol(base_asset)
        delimiter = "/"
        if "-" in market:
            delimiter = "-"

        # Combine the unwrapped base asset with the quote asset
        return f"{unwrapped_base}{delimiter}{quote_asset}"

    def clean_markets(self, markets: dict) -> dict:
        """
        Removes all non-compliant markets, those exceeding 12 characters in length,
        and those not containing a '-' or '/'.
        """
        allowed_chars_pattern = re.compile(r"^[a-zA-Z0-9/-]{1,12}$")
        must_have_chars = {"-", "/"}

        cleaned_markets = []

        for market in markets:
            if allowed_chars_pattern.match(market) and any(
                char in market for char in must_have_chars
            ):
                cleaned_markets.append(market)
            else:
                self.logger.info(
                    f"Invalid market format, length, or missing required characters found and removed: {market}"
                )
        self.logger.debug(f"Cleaned markets: {cleaned_markets}")
        return cleaned_markets

    def get_base(self, pair_string: str) -> str:
        """Returns the base asset. From SOL/USDT that is SOL."""
        match = re.match(r"([A-Za-z]+)[/-]", pair_string)
        if match:
            base_asset = match.group(1)
            return base_asset
        else:
            raise ValueError("Invalid pair string format")

    def get_quote(self, pair_string: str) -> str:
        """Returns the quote asset. From SOL/USDT that is USDT."""
        match = re.match(r"[A-Za-z]+[/-]([A-Za-z]+)", pair_string)
        if match:
            quote_asset = match.group(1)
            return quote_asset
        else:
            # Fall back to default quote asset if not found
            return self.default_quote_asset

    def format_pair(self, pair_string: str) -> str:
        raise NotImplementedError("This method should be implemented by subclasses.")

    def format_set_of_pairs(self, pairs: Set[str]) -> Set[str]:
        return {self.format_pair(pair) for pair in pairs}

    def format_pair_default(self, pair_string: str) -> str:
        """Will format the pair string to a slash-format. Mainly used for the ccxt LIB."""
        # Regex pattern to match 'BTCUSDT', 'BTC/USDT', and 'BTC-USDT'
        pattern = r"([A-Za-z]+)[/-]?(USDT)$"
        match = re.match(pattern, pair_string)

        if match:
            base_asset, quote_asset = match.groups()
            formatted_pair = f"{base_asset}/{quote_asset}"
            return formatted_pair
        else:
            self.logger.error(f"Invalid pair format: {pair_string}")
            raise ValueError(f"Invalid pair format: {pair_string}")

    def format_to_dash(self, pair_string: str) -> str:
        """Takes any combination of a market pair and replaces the divider by a dash (-)."""
        match = re.match(r"([A-Za-z0-9]+)[/-]?([A-Za-z0-9]+)", pair_string)        
        if match is None:
            raise ValueError(f"Invalid market pair format: '{pair_string}'")
        base_asset, _ = match.groups()
        return f"{base_asset}-{self.default_quote_asset}"

    def format_to_slash(self, pair_string: str) -> str:
        """Takes any combinatin of a market pair and replaces the divider by a front slash (/)."""
        match = re.match(r"([A-Za-z0-9]+)[/-]?([A-Za-z0-9]+)", pair_string)        
        if match is None:
            raise ValueError(f"Invalid market pair format: '{pair_string}'")
        base_asset, _ = match.groups()
        return f"{base_asset}/{self.default_quote_asset}"

    def format_set_to_dash(self, markets: Set[str]) -> Set[str]:
        """Takes a set of market pairs and formats each to dash notation."""
        return {self.format_to_dash(pair) for pair in markets}

    def format_set_to_slash(self, markets: Set[str]) -> Set[str]:
        """Takes a set of market pairs and formats each to front slash notation."""
        return {self.format_to_slash(pair) for pair in markets}


class BinanceFormatter(AssetFormatter):
    def format_pair(self, pair_string: str) -> str:
        match = re.match(r"([A-Za-z]+)[/-]?([A-Za-z]+)", pair_string)
        base_asset, _ = match.groups()
        return f"{base_asset}-{self.default_quote_asset}"


class BybitFormatter(AssetFormatter):
    def format_pair(self, pair_string: str) -> str:
        match = re.match(r"([A-Za-z]+)[/-]?([A-Za-z]+)", pair_string)
        base_asset, _ = match.groups()
        # Assuming Bybit format is slightly different, for example
        return f"{base_asset}_{self.default_quote_asset}"



import base64
import json
import os
from tksessentials.constants import DEFAULT_ENCODING
from builtins import bytes
from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
from cryptography.hazmat.primitives import padding
from cryptography.hazmat.backends import default_backend
from tksessentials import global_logger, utils

def get_secret_key() -> str:
    """Returns the general encryption key to encrypt data."""
    key = os.getenv("ENCRYPTION_KEY")
    if not key:
        raise ValueError("ENCRYPTION_KEY environment variable is not set or empty.")
    return key

def get_JWT_secret() -> str:
    """Returns JWT secret."""
    jwt_secret = os.getenv("JWT_SECRET")
    if not jwt_secret:
        raise ValueError("JWT_SECRET environment variable is not set or empty.")
    return jwt_secret

def get_AES_secret() -> bytes:
    """Returns AES secret."""
    aes_secret = bytes(os.getenv("AES_SECRET"), DEFAULT_ENCODING)
    if not aes_secret:
        raise ValueError("AES_SECRET environment variable is not set or empty.")
    return aes_secret


class Crypto:
    def __init__(self):
        self.backend = default_backend()
        self.key = base64.urlsafe_b64encode(get_AES_secret())[:32]
        self.encryptor = Cipher(algorithms.AES(self.key), modes.ECB(), self.backend).encryptor()
        self.decryptor = Cipher(algorithms.AES(self.key), modes.ECB(), self.backend).decryptor()

    def encrypt(self, value: str) -> bytes:
        byte_value = bytes(value, DEFAULT_ENCODING)
        padder = padding.PKCS7(algorithms.AES(self.key).block_size).padder()
        padded_data = padder.update(byte_value) + padder.finalize()
        encrypted_text = self.encryptor.update(padded_data) + self.encryptor.finalize()
        return base64.urlsafe_b64encode(encrypted_text)

    def encrypt_as_text(self, value) -> str:
        return str(self.encrypt(value), encoding=DEFAULT_ENCODING)

    def decrypt(self, value: str) -> bytes:
        byte_value = base64.urlsafe_b64decode(bytes(value, DEFAULT_ENCODING))
        padder = padding.PKCS7(algorithms.AES(self.key).block_size).unpadder()
        decrypted_data = self.decryptor.update(byte_value)
        unpadded = padder.update(decrypted_data) + padder.finalize()
        return unpadded

    def decrypt_as_text(self, value) -> str:
        return str(self.decrypt(value), encoding=DEFAULT_ENCODING)



from email_validator import EmailSyntaxError
import pytest
from tksessentials import validators

@pytest.mark.parametrize("email", ["john.doe@gmail.com"])
def test_email_address(email):
    try:
        validators.validate_email(email)
    except EmailSyntaxError:
        pytest.fail("validate_email() raised exception unexpectedly!")    

@pytest.mark.parametrize("email", ["name@com", "@domain.com", "name@domain", "firstname.name@DOMAIN-THAT-PROBABLY-WILL-NEVER-EXIST-TEST-0123.com"])
def test_wrong_email_addresses(email):
    with pytest.raises(Exception): 
        validators.validate_email(email)


@pytest.mark.parametrize("ip_address", ["100.128.0.0"])
def test_ip_addresses(ip_address: str):
    try:
        validators.validate_ip_address(ip_address)
    except ValueError:
        pytest.fail("validate_ip_address() raised exception unexpectedly!")      

@pytest.mark.parametrize("ip_address", [
    None,
    "127 .0.0.1",
    "999.255.255.255",
    "100.128.0.0/222"])
def test_wrong_ip_addresses(ip_address: str):
    with pytest.raises(Exception): 
        validators.validate_ip_address(ip_address)



import unittest
from unittest.mock import patch, Mock

from tksessentials.database import is_ksqldb_available

class TestKsqlDBAvailability(unittest.TestCase):

    @patch('tksessentials.database.httpx.get')
    def test_ksqldb_available(self, mock_httpx_get):
        # Mocking a successful response from ksqlDB
        mock_response = Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = {'KsqlServerInfo': {'serverStatus': 'RUNNING'}}
        mock_httpx_get.return_value = mock_response

        result = is_ksqldb_available()
        print(result)
        self.assertTrue(result)

    @patch('tksessentials.database.httpx.get')
    def test_ksqldb_unavailable(self, mock_httpx_get):
        # Mocking a response with NOT_RUNNING status from ksqlDB
        mock_response = Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = {'KsqlServerInfo': {'serverStatus': 'NOT_RUNNING'}}
        mock_httpx_get.return_value = mock_response

        result = is_ksqldb_available()
        print(result)
        self.assertFalse(result)

    @patch('tksessentials.database.httpx.get')
    def test_ksqldb_no_response(self, mock_httpx_get):
        # Mocking no response from ksqlDB
        mock_response = Mock()
        mock_response.status_code = 500
        mock_httpx_get.return_value = mock_response

        result = is_ksqldb_available()
        print(result)
        self.assertFalse(result)

if __name__ == '__main__':
    unittest.main()



import pytest
from tksessentials.security import Crypto

AES_SECRET = "ZufDdKmoYgBv272G0DQWqz8Ng9ewM+IMGIMzkRQUoVNujiCHfdD4EULwXtn5fvwL"

@pytest.mark.parametrize("message", ["test", "123908234ÖÄ$ASDFdd", "äjklöèü"])
def test_encrypt_and_decrypt_text_message(monkeypatch, message):
    monkeypatch.setenv("AES_SECRET", AES_SECRET)
    crypto = Crypto()

    encrypted_message = crypto.encrypt_as_text(message)
    decrypted_message = crypto.decrypt_as_text(encrypted_message)

    assert decrypted_message == message

@pytest.mark.parametrize("message", ["firstname.name@domain.com"])
def test_encrypt_and_decrypt_email_address(monkeypatch, message):
    monkeypatch.setenv("AES_SECRET", AES_SECRET)
    crypto = Crypto()

    encrypted_message = crypto.encrypt_as_text(message)
    decrypted_message = crypto.decrypt_as_text(encrypted_message)

    assert decrypted_message == message

# FAILY - NEEDS TO BE FIXED
# @pytest.mark.parametrize("ip_address, blocking_reason", [("155.255.452.55", BlockedIpReasonType.EXCESSIVE_FAILED_LOGIN_ATTEMPTS)])
# def test_block_ip_address(ip_address: str, blocking_reason: BlockedIpReasonType):
#     ipSecurity = IPSecurity()

#     assert ipSecurity.block_ip(ip_address, blocking_reason) is True
#     assert ipSecurity.is_ip_blocked(ip_address) is True
#     assert ipSecurity.unblock_ip(ip_address) is True
#     assert ipSecurity.is_ip_blocked(ip_address) is False
